"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cheerio_1 = tslib_1.__importDefault(require("cheerio"));
const fs = tslib_1.__importStar(require("fs-extra"));
const os_1 = require("os");
const path = tslib_1.__importStar(require("path"));
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
const tough = require("tough-cookie");
const tough_cookie_filestore_1 = tslib_1.__importDefault(require("tough-cookie-filestore"));
const url = tslib_1.__importStar(require("url"));
const utils_1 = require("./utils");
const defaultCookiePath = path.join(os_1.homedir(), '.hackmd', 'cookies.json');
const defaultConfig = {
    cookiePath: defaultCookiePath,
    serverUrl: 'https://hackmd.io',
    enterprise: true
};
function encodeFormComponent(form) {
    return Object.entries(form).map(([key, value]) => `${encodeURIComponent(key)}=${encodeURIComponent(value)}`).join('&');
}
var ExportType;
(function (ExportType) {
    ExportType[ExportType["PDF"] = 0] = "PDF";
    ExportType[ExportType["SLIDE"] = 1] = "SLIDE";
    ExportType[ExportType["MD"] = 2] = "MD";
    ExportType[ExportType["HTML"] = 3] = "HTML";
})(ExportType = exports.ExportType || (exports.ExportType = {}));
/**
 * codimd API Client
 */
class API {
    constructor(config = {}) {
        const { serverUrl, cookiePath, enterprise } = utils_1.defaults(config, defaultConfig);
        fs.ensureFileSync(cookiePath);
        const jar = new tough_cookie_filestore_1.default(cookiePath);
        const fetch = require('fetch-cookie')(node_fetch_1.default, new tough.CookieJar(jar));
        this._fetch = fetch;
        this.serverUrl = serverUrl;
        this.enterprise = enterprise;
    }
    async login(email, password) {
        const response = await this.fetch(`${this.serverUrl}/login`, {
            method: 'post',
            body: encodeFormComponent({ email, password }),
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
            })
        });
        return response.status === 200;
    }
    async loginLdap(username, password) {
        const response = await this.fetch(`${this.serverUrl}/auth/ldap`, {
            method: 'post',
            body: encodeFormComponent({ username, password }),
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
            })
        });
        return response.status === 200;
    }
    async logout() {
        const response = await this.fetch(`${this.serverUrl}/logout`, {
            method: this.enterprise ? 'POST' : 'GET',
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
            })
        });
        return response.status === 200;
    }
    async isLogin() {
        const response = await this.fetch(`${this.serverUrl}/me`);
        return response.status === 200;
    }
    async getMe() {
        const response = await this.fetch(`${this.serverUrl}/me`);
        return response.json();
    }
    async getHistory() {
        const response = await this.fetch(`${this.serverUrl}/history`);
        return response.json();
    }
    async newNote(body) {
        let response;
        if (this.enterprise) {
            response = await this.fetch(`${this.serverUrl}/new`, {
                method: 'POST',
                body: encodeFormComponent({ content: body }),
                headers: await this.wrapHeaders({
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                })
            });
        }
        else {
            const contentType = 'text/markdown;charset=UTF-8';
            response = await this.fetch(`${this.serverUrl}/new`, {
                method: 'POST',
                body,
                headers: {
                    'Content-Type': contentType
                }
            });
        }
        if (response.status === 200) {
            return response.url;
        }
        else {
            throw new Error('Create note failed');
        }
    }
    async export(noteId, type, output) {
        let res;
        switch (type) {
            case ExportType.PDF:
                res = await this.fetch(`${this.serverUrl}/${noteId}/pdf`);
                break;
            case ExportType.HTML:
                res = await this.fetch(`${this.serverUrl}/s/${noteId}`);
                break;
            case ExportType.SLIDE:
                res = await this.fetch(`${this.serverUrl}/${noteId}/slide`);
                break;
            case ExportType.MD:
            default:
                res = await this.fetch(`${this.serverUrl}/${noteId}/download`);
        }
        return this.downloadFile(res, output);
    }
    async downloadFile(res, output) {
        const fileStream = fs.createWriteStream(output);
        await new Promise((resolve, reject) => {
            res.body.pipe(fileStream);
            res.body.on('error', (err) => {
                reject(err);
            });
            fileStream.on('finish', function () {
                resolve();
            });
        });
    }
    get fetch() {
        return this._fetch;
    }
    get domain() {
        return url.parse(this.serverUrl).host;
    }
    async wrapHeaders(headers) {
        if (this.enterprise) {
            const csrf = await this.loadCSRFToken();
            return Object.assign(Object.assign({}, headers), { 'X-XSRF-Token': csrf });
        }
        else {
            return headers;
        }
    }
    async loadCSRFToken() {
        const html = await this.fetch(`${this.serverUrl}`).then(r => r.text());
        const $ = cheerio_1.default.load(html);
        return $('meta[name="csrf-token"]').attr('content') || '';
    }
}
exports.default = API;
